package org.ow2.util.pool.impl.enhanced.impl.waitcontrol;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.locks.Condition;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.WaiterInterruptedException;
import org.ow2.util.pool.impl.enhanced.internal.conditionWaitPart.IConditionWaitPart;

/**
 * @author Gael Lalire
 */
public class InfiniteWaitControl implements IWaitControl {

    private List<WaitAuthorization> waitAuthorizations;

    public InfiniteWaitControl() {
        waitAuthorizations = new ArrayList<WaitAuthorization>();
    }

    protected boolean canWait() {
        for (WaitAuthorization waitAuthorization : waitAuthorizations) {
            if (!waitAuthorization.isWaitAllowed()) {
                return false;
            }
        }
        return true;
    }

    public void pushWaitAuthorization(final WaitAuthorization waitAuthorization) {
        waitAuthorizations.add(0, waitAuthorization);
    }

    public void popWaitAuthorization() {
        waitAuthorizations.remove(0);
    }

    /**
     * @param condition a condition
     * @return false if timeout, true if notify
     * @throws InterruptedException
     */
    public boolean waitOnCondition(final Condition condition) throws InterruptedException, WaiterInterruptedException {
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
        condition.await();
        return true;
    }

    public boolean waitOnMutex(final Object mutex) throws InterruptedException, WaiterInterruptedException {
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
        mutex.wait();
        return true;
    }

    public boolean canContinueToWait() {
        return true;
    }

    public boolean waitOnConditionWaitPart(final IConditionWaitPart condition) throws InterruptedException,
            WaiterInterruptedException {
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
        condition.await();
        return true;
    }

    public void verifyInterrupted() throws WaiterInterruptedException {
        if (!canWait()) {
            throw new WaiterInterruptedException();
        }
    }

}
